@@ -76,28 +76,28 @@ totem_pl_parser_write_pls (TotemPlParser *parser, GtkTreeModel *model,
 
 	for (i = 1; i <= num_entries_total; i++) {
 		GtkTreeIter iter;
-		char *url, *title, *relative;
+		char *uri, *title, *relative;
 		GFile *file;
 		gboolean custom_title;
 
 		if (gtk_tree_model_iter_nth_child (model, &iter, NULL, i - 1) == FALSE)
 			continue;
 
-		func (model, &iter, &url, &title, &custom_title, user_data);
+		func (model, &iter, &uri, &title, &custom_title, user_data);
 
-		file = g_file_new_for_uri (url);
+		file = g_file_new_for_uri (uri);
 		if (totem_pl_parser_scheme_is_ignored (parser, file) != FALSE) {
-			g_free (url);
+			g_free (uri);
 			g_free (title);
 			g_object_unref (file);
 			continue;
 		}
 		g_object_unref (file);
 
-		relative = totem_pl_parser_relative (output, url);
-		buf = g_strdup_printf ("File%d=%s\n", i, relative ? relative : url);
+		relative = totem_pl_parser_relative (output, uri);
+		buf = g_strdup_printf ("File%d=%s\n", i, relative ? relative : uri);
 		g_free (relative);
-		g_free (url);
+		g_free (uri);
 		success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), buf, error);
 		g_free (buf);
 		if (success == FALSE) {
@@ -122,28 +122,42 @@ totem_pl_parser_write_pls (TotemPlParser *parser, GtkTreeModel *model,
 	return TRUE;
 }
 
+static char *
+ensure_utf8_valid (char *input) 
+{
+	char *utf8_valid;
+
+	utf8_valid = g_strdup (input);
+
+	if (!g_utf8_validate (utf8_valid, -1, NULL)) {
+		gint i;
+
+		for (i = 0; i < g_utf8_strlen (utf8_valid, -1); i++) {
+			gunichar c;
+			c = g_utf8_get_char_validated (&utf8_valid[i], -1);
+			if (c > 127) {
+				utf8_valid[i] = '?';
+			}
+		}
+	}
+	return utf8_valid;
+}
+
 TotemPlParserResult
 totem_pl_parser_add_pls_with_contents (TotemPlParser *parser,
 				       GFile *file,
 				       GFile *_base_file,
-				       const char *contents)
+				       const char *contents,
+				       TotemPlParseData *parse_data)
 {
 	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 	GFile *base_file;
 	char **lines;
 	int i, num_entries;
-	char *split_char, *playlist_title;
-	gboolean dos_mode = FALSE;
+	char *playlist_title;
 	gboolean fallback;
 
-	/* figure out whether we're a unix pls or dos pls */
-	if (strstr(contents,"\x0d") == NULL) {
-		split_char = "\n";
-	} else {
-		split_char = "\x0d\n";
-		dos_mode = TRUE;
-	}
-	lines = g_strsplit (contents, split_char, 0);
+	lines = g_strsplit_set (contents, "\r\n", 0);
 
 	/* [playlist] */
 	i = 0;
@@ -160,10 +174,10 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser,
 	}
 
 	playlist_title = totem_pl_parser_read_ini_line_string (lines,
-			"X-GNOME-Title", dos_mode);
+			"X-GNOME-Title");
 
 	if (playlist_title != NULL) {
-		totem_pl_parser_add_url (parser,
+		totem_pl_parser_add_uri (parser,
 					 TOTEM_PL_PARSER_FIELD_IS_PLAYLIST, TRUE,
 					 TOTEM_PL_PARSER_FIELD_FILE, file,
 					 TOTEM_PL_PARSER_FIELD_TITLE, playlist_title,
@@ -208,10 +222,10 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser,
 		/* Genre is our own little extension */
 		genre_key = g_strdup_printf ("genre%d", i);
 
-		file_str = totem_pl_parser_read_ini_line_string (lines, (const char*)file_key, dos_mode);
-		title = totem_pl_parser_read_ini_line_string (lines, (const char*)title_key, dos_mode);
-		genre = totem_pl_parser_read_ini_line_string (lines, (const char*)genre_key, dos_mode);
-		length = totem_pl_parser_read_ini_line_string (lines, (const char*)length_key, dos_mode);
+		file_str = totem_pl_parser_read_ini_line_string (lines, (const char*)file_key);
+		title = totem_pl_parser_read_ini_line_string (lines, (const char*)title_key);
+		genre = totem_pl_parser_read_ini_line_string (lines, (const char*)genre_key);
+		length = totem_pl_parser_read_ini_line_string (lines, (const char*)length_key);
 
 		g_free (file_key);
 		g_free (title_key);
@@ -225,22 +239,22 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser,
 			continue;
 		}
 
-		fallback = parser->priv->fallback;
-		if (parser->priv->recurse)
-			parser->priv->fallback = FALSE;
+		fallback = parse_data->fallback;
+		if (parse_data->recurse)
+			parse_data->fallback = FALSE;
 
 		/* Get the length, if it's negative, that means that we have a stream
 		 * and should push the entry straight away */
 		if (length != NULL)
-			length_num = totem_pl_parser_parse_duration (length, parser->priv->debug);
+			length_num = totem_pl_parser_parse_duration (length, totem_pl_parser_is_debugging_enabled (parser));
 
 		if (strstr (file_str, "://") != NULL || file_str[0] == G_DIR_SEPARATOR) {
 			GFile *target;
 
 			target = g_file_new_for_commandline_arg (file_str);
-			if (length_num < 0 || totem_pl_parser_parse_internal (parser, target, NULL) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
-				totem_pl_parser_add_url (parser,
-							 TOTEM_PL_PARSER_FIELD_URL, file_str,
+			if (length_num < 0 || totem_pl_parser_parse_internal (parser, target, NULL, parse_data) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
+				totem_pl_parser_add_uri (parser,
+							 TOTEM_PL_PARSER_FIELD_URI, file_str,
 							 TOTEM_PL_PARSER_FIELD_TITLE, title,
 							 TOTEM_PL_PARSER_FIELD_GENRE, genre,
 							 TOTEM_PL_PARSER_FIELD_DURATION, length,
@@ -249,12 +263,14 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser,
 			g_object_unref (target);
 		} else {
 			GFile *target;
+			char *utf8_filename;
 
-			target = g_file_get_child_for_display_name (base_file, file_str, NULL);
+			utf8_filename = ensure_utf8_valid (file_str);
+			target = g_file_get_child_for_display_name (base_file, utf8_filename, NULL);
+			g_free (utf8_filename);
 
-			if (length_num < 0 || totem_pl_parser_parse_internal (parser, target, base_file) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
-
-				totem_pl_parser_add_url (parser,
+			if (length_num < 0 || totem_pl_parser_parse_internal (parser, target, base_file, parse_data) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
+				totem_pl_parser_add_uri (parser,
 							 TOTEM_PL_PARSER_FIELD_FILE, target,
 							 TOTEM_PL_PARSER_FIELD_TITLE, title,
 							 TOTEM_PL_PARSER_FIELD_GENRE, genre,
@@ -265,7 +281,7 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser,
 			g_object_unref (target);
 		}
 
-		parser->priv->fallback = fallback;
+		parse_data->fallback = fallback;
 		g_free (file_str);
 		g_free (title);
 		g_free (genre);
@@ -288,6 +304,7 @@ TotemPlParserResult
 totem_pl_parser_add_pls (TotemPlParser *parser,
 			 GFile *file,
 			 GFile *base_file,
+			 TotemPlParseData *parse_data,
 			 gpointer data)
 {
 	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
@@ -302,7 +319,7 @@ totem_pl_parser_add_pls (TotemPlParser *parser,
 		return TOTEM_PL_PARSER_RESULT_SUCCESS;
 	}
 
-	retval = totem_pl_parser_add_pls_with_contents (parser, file, base_file, contents);
+	retval = totem_pl_parser_add_pls_with_contents (parser, file, base_file, contents, parse_data);
 	g_free (contents);
 
 	return retval;
