@@ -57,8 +57,7 @@ totem_pl_parser_is_quicktime (const char *data, gsize len)
 	if (g_str_has_prefix (data, "SMILtext") != FALSE)
 		return QUICKTIME_META_MIME_TYPE;
 
-	if (memmem (data, len,
-		    QT_NEEDLE, strlen (QT_NEEDLE)) != NULL)
+	if (g_strstr_len (data, len, QT_NEEDLE) != NULL)
 		return QUICKTIME_META_MIME_TYPE;
 
 	return NULL;
@@ -70,40 +69,42 @@ static TotemPlParserResult
 totem_pl_parser_add_quicktime_rtsptext (TotemPlParser *parser,
 					GFile *file,
 					GFile *base_file,
+					TotemPlParseData *parse_data,
 					gpointer data)
 {
 	char *contents = NULL;
-	gboolean dos_mode = FALSE;
-	char *volume, *autoplay, *rtspurl;
+	char *volume, *autoplay, *rtspuri;
 	gsize size;
 	char **lines;
 
 	if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
-	if (strstr(contents,"\x0d") != NULL)
-		dos_mode = TRUE;
-
-	lines = g_strsplit (contents, dos_mode ? "\x0d\n" : "\n", 0);
+	lines = g_strsplit_set (contents, "\r\n", 0);
 
 	volume = totem_pl_parser_read_ini_line_string_with_sep
-		(lines, "volume", dos_mode, "=");
+		(lines, "volume", "=");
 	autoplay = totem_pl_parser_read_ini_line_string_with_sep
-		(lines, "autoplay", dos_mode, "=");
+		(lines, "autoplay", "=");
 
-	rtspurl = g_strdup (lines[0] + strlen ("RTSPtext"));
-	if (rtspurl[0] == '\0') {
-		g_free (rtspurl);
-		rtspurl = g_strdup (lines[1]);
+	rtspuri = g_strdup (lines[0] + strlen ("RTSPtext"));
+	if (rtspuri[0] == '\0') {
+		char **line;
+		g_free (rtspuri);
+
+		for (line = lines; line && *line[0] == '\0'; line++);
+		if (line == NULL)
+			return TOTEM_PL_PARSER_RESULT_ERROR;
+		rtspuri = g_strdup (*line);
 	}
-	g_strstrip (rtspurl);
+	g_strstrip (rtspuri);
 
-	totem_pl_parser_add_url (parser,
-				 TOTEM_PL_PARSER_FIELD_URL, rtspurl,
+	totem_pl_parser_add_uri (parser,
+				 TOTEM_PL_PARSER_FIELD_URI, rtspuri,
 				 TOTEM_PL_PARSER_FIELD_VOLUME, volume,
 				 TOTEM_PL_PARSER_FIELD_AUTOPLAY, autoplay,
 				 NULL);
-	g_free (rtspurl);
+	g_free (rtspuri);
 	g_free (volume);
 	g_free (autoplay);
 	g_strfreev (lines);
@@ -114,17 +115,19 @@ totem_pl_parser_add_quicktime_rtsptext (TotemPlParser *parser,
 static TotemPlParserResult
 totem_pl_parser_add_quicktime_metalink (TotemPlParser *parser,
 					GFile *file,
-					GFile *base_file, gpointer data)
+					GFile *base_file,
+					TotemPlParseData *parse_data,
+					gpointer data)
 {
 	xml_node_t *doc, *node;
 	gsize size;
 	char *contents;
-	const char *item_url, *autoplay;
+	const char *item_uri, *autoplay;
 	gboolean found;
 
 	if (g_str_has_prefix (data, "RTSPtext") != FALSE
 			|| g_str_has_prefix (data, "rtsptext") != FALSE) {
-		return totem_pl_parser_add_quicktime_rtsptext (parser, file, base_file, data);
+		return totem_pl_parser_add_quicktime_rtsptext (parser, file, base_file, parse_data, data);
 	}
 	if (g_str_has_prefix (data, "SMILtext") != FALSE) {
 		char *contents;
@@ -145,8 +148,8 @@ totem_pl_parser_add_quicktime_metalink (TotemPlParser *parser,
 	if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
-	xml_parser_init (contents, size, XML_PARSER_CASE_INSENSITIVE);
-	if (xml_parser_build_tree_with_options (&doc, XML_PARSER_RELAXED | XML_PARSER_MULTI_TEXT) < 0) {
+	doc = totem_pl_parser_parse_xml_relaxed (contents, size);
+	if (doc == NULL) {
 		g_free (contents);
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 	}
@@ -177,8 +180,8 @@ totem_pl_parser_add_quicktime_metalink (TotemPlParser *parser,
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 	}
 
-	item_url = xml_parser_get_property (doc, "src");
-	if (!item_url) {
+	item_uri = xml_parser_get_property (doc, "src");
+	if (!item_uri) {
 		xml_parser_free_tree (doc);
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 	}
@@ -188,8 +191,8 @@ totem_pl_parser_add_quicktime_metalink (TotemPlParser *parser,
 	if (autoplay == NULL)
 		autoplay = "true";
 
-	totem_pl_parser_add_url (parser,
-				 TOTEM_PL_PARSER_FIELD_URL, item_url,
+	totem_pl_parser_add_uri (parser,
+				 TOTEM_PL_PARSER_FIELD_URI, item_uri,
 				 TOTEM_PL_PARSER_FIELD_AUTOPLAY, autoplay,
 				 NULL);
 	xml_parser_free_tree (doc);
@@ -201,18 +204,13 @@ TotemPlParserResult
 totem_pl_parser_add_quicktime (TotemPlParser *parser,
 			       GFile *file,
 			       GFile *base_file,
+			       TotemPlParseData *parse_data,
 			       gpointer data)
 {
-	if (data == NULL || totem_pl_parser_is_quicktime (data, strlen (data)) == NULL) {
-		char *url;
+	if (data == NULL || totem_pl_parser_is_quicktime (data, strlen (data)) == NULL)
+		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
-		url = g_file_get_uri (file);
-		totem_pl_parser_add_one_url (parser, url, NULL);
-		g_free (url);
-		return TOTEM_PL_PARSER_RESULT_SUCCESS;
-	}
-
-	return totem_pl_parser_add_quicktime_metalink (parser, file, base_file, data);
+	return totem_pl_parser_add_quicktime_metalink (parser, file, base_file, parse_data, data);
 }
 
 #endif /* !TOTEM_PL_PARSER_MINI */
