@@ -73,12 +73,12 @@ parse_smil_entry (TotemPlParser *parser,
 			copyright = xml_parser_get_property (node, "copyright");
 
 			if (uri != NULL) {
+				char *resolved_uri;
 				GFile *resolved;
 
-				if (base_file != NULL && strstr (uri, "://") == NULL)
-					resolved = g_file_resolve_relative_path (base_file, uri);
-				else
-					resolved = g_file_new_for_uri (uri);
+				resolved_uri = totem_pl_parser_resolve_uri (base_file, uri);
+				resolved = g_file_new_for_uri (resolved_uri);
+				g_free (resolved_uri);
 
 				totem_pl_parser_add_uri (parser,
 							 TOTEM_PL_PARSER_FIELD_FILE, resolved,
@@ -172,7 +172,9 @@ totem_pl_parser_add_smil_with_doc (TotemPlParser *parser, GFile *file,
 TotemPlParserResult
 totem_pl_parser_add_smil (TotemPlParser *parser,
 			  GFile *file,
-			  GFile *base_file, gpointer data)
+			  GFile *base_file,
+			  TotemPlParseData *parse_data,
+			  gpointer data)
 {
 	char *contents;
 	gsize size;
@@ -197,8 +199,8 @@ totem_pl_parser_add_smil_with_data (TotemPlParser *parser,
 	xml_node_t* doc;
 	TotemPlParserResult retval;
 
-	xml_parser_init (contents, size, XML_PARSER_CASE_INSENSITIVE);
-	if (xml_parser_build_tree_with_options (&doc, XML_PARSER_RELAXED | XML_PARSER_MULTI_TEXT) < 0)
+	doc = totem_pl_parser_parse_xml_relaxed (contents, size);
+	if (doc == NULL)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	retval = totem_pl_parser_add_smil_with_doc (parser, file, base_file, doc);
